package de.fraunhofer.iem.dataprovider.kpi.enumeration

import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeChildResponseDto
import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeResponseDto

enum class KpiKind {
    // Raw Value KPIs
    CHECKED_IN_BINARIES {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to assess the compliance to the OpenCoDE " +
                    "platform guidelines in regards of not checking in binaries.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "No Checked in Binaries"
        }
    },
    NUMBER_OF_COMMITS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Total number of commits on the default branch of the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Number of Commits"
        }
    },
    VULNERABILITY_SCORE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            // TODO: in order to provide more detailed information here we need to query the ORT api
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                displayValue = "${value.toDouble() / 10}",
                description = "A vulnerability with this score was " +
                    "found in the projects dependencies. Further information are not disclosed here.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Vulnerability Score"
        }
    },
    NUMBER_OF_SIGNED_COMMITS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Total number of signed and verified commits on the default branch of the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Number of Signed Commits"
        }
    },
    IS_DEFAULT_BRANCH_PROTECTED {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to assess compliance with a standard development process." +
                    " For this purpose, it is examined whether the standard development" +
                    " branch is protected against unintentional changes.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Default Branch Protection"
        }
    },
    SECRETS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to assess the security of the software provided. " +
                    "For this purpose, it is checked at code level whether actual secrets " +
                    "are revealed within the code. There is a possibility that discovered potential secrets " +
                    "are no actual secrets. In future updates, the developers will be allowed " +
                    "to contradict the analysis results.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Public Secrets"
        }
    },
    SAST_USAGE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to approximate the usage of SAST tools during development.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "SAST Usage"
        }
    },
    COMMENTS_IN_CODE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to approximate the state of documentation of the code by measuring the " +
                    "amount of code comments compared to the lines of code.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Comments in Code"
        }
    },
    DOCUMENTATION_INFRASTRUCTURE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to assess compliance with a common development process and" +
                    " certain software quality standards. For this purpose, it is checked whether " +
                    "the repository provides documentation for the software made available. It should " +
                    "be noted that this is not a sufficient criterion, since in this case only the existence " +
                    "of certain folders or wiki pages is checked.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Existence of Documentation Infrastructure"
        }
    },

    // Calculated KPIs
    SIGNED_COMMITS_RATIO {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Used to assess compliance with a common and " +
                    "transparent development process. It is desirable that all commits " +
                    "are signed by their authors. Therefore, the ratio of signed commits " +
                    "to all commits is determined to calculate this metric.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Commit Signature Ratio"
        }
    },
    INTERNAL_QUALITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses software quality from a developer perspective" +
                    " and includes metrics for code quality, maintainability, readability and testability.",
                children = children,
                isEmpty = isEmpty,
                order = 3
            )
        }

        override fun getName(): String {
            return "Internal Quality"
        }
    },
    EXTERNAL_QUALITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses the software quality from the user's perspective " +
                    "and includes metrics for the fulfillment of functional requirements, user-friendliness, " +
                    "reliability and performance.",
                children = children,
                isEmpty = isEmpty,
                order = 3
            )
        }

        override fun getName(): String {
            return "External Quality"
        }
    },
    PROCESS_COMPLIANCE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses the development process of the software provided." +
                    " For this purpose, the development process traceable in the repository is" +
                    " compared with common development standards to enable an assessment.",
                children = children,
                isEmpty = isEmpty,
                order = 4
            )
        }

        override fun getName(): String {
            return "Process Compliance Score"
        }
    },
    PROCESS_TRANSPARENCY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses the transparency resp. traceability of the development " +
                    "process of the provided software for external parties. For this purpose," +
                    " various analyzes are performed that assess the availability of information " +
                    "about the software development process within the repository.",
                children = children,
                isEmpty = isEmpty,
                order = 5
            )
        }

        override fun getName(): String {
            return "Process Transparency Score"
        }
    },
    SECURITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses the security of the software provided. For this purpose, " +
                    "various security-relevant analyzes are carried out, which, among other things," +
                    " check the external dependencies or the code for vulnerabilities.",
                children = children,
                isEmpty = isEmpty,
                order = 2
            )
        }

        override fun getName(): String {
            return "Security Score"
        }
    },
    MAXIMAL_VULNERABILITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Serves to assess the security of the software provided. For this purpose, " +
                    "the external dependencies of the software are analyzed for vulnerabilities. The assessment" +
                    " is performed by selecting the most critical vulnerability according to the " +
                    "Common Vulnerability " +
                    "Scoring System (CVSS) for assessing IT security vulnerabilities among all " +
                    "identified external dependencies." +
                    "This score is calculated by the following formula: 100 - (max(CVSS score) * 10). " +
                    "Thus, a lower value indicates a more critical vulnerability.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Maximal Dependency Vulnerability"
        }
    },
    DOCUMENTATION {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "This score describes the approximated availability of documentation in the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Documentation"
        }
    },

    // ROOT
    ROOT {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = this.getName(),
                description = "Assesses the project resp. the provided software in the aspects of" +
                    " maturity (based on quality, security and usability aspects) as well as development process.",
                isRoot = true,
                children = children,
                isEmpty = isEmpty
            )
        }

        override fun getName(): String {
            return "Project Score"
        }
    };

    abstract fun toViewModel(
        value: Int,
        children: List<KPITreeChildResponseDto>,
        isEmpty: Boolean
    ): KPITreeResponseDto

    abstract fun getName(): String
}
